@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -94,25 +94,6 @@ function countRequest($str = null) {
 	}
 }
 
-/************ COOKIES ************/
-function get_cookie($name, $default_value = null) {
-	if (isset($_COOKIE[$name])) {
-		return $_COOKIE[$name];
-	}
-
-	return $default_value;
-}
-
-function zbx_setcookie($name, $value, $time = null) {
-	setcookie($name, $value, isset($time) ? $time : 0, CSession::getDefaultCookiePath(), null, HTTPS, true);
-	$_COOKIE[$name] = $value;
-}
-
-function zbx_unsetcookie($name) {
-	zbx_setcookie($name, null, -99999);
-	unset($_COOKIE[$name]);
-}
-
 /************* DATE *************/
 function getMonthCaption($num) {
 	switch ($num) {
@@ -554,7 +535,7 @@ function convertUnitsS($value, $ignore_millisec = false) {
 		foreach ([
 			'days' => SEC_PER_DAY,
 			'hours' => SEC_PER_HOUR,
-			'minutes' => SEC_PER_MIN,
+			'minutes' => SEC_PER_MIN
 		] as $part => $sec_per_part) {
 			$v = floor($value_abs_int / $sec_per_part);
 			if ($v > 0) {
@@ -604,8 +585,8 @@ function convertUnitsS($value, $ignore_millisec = false) {
 
 	$result = [];
 
-	foreach (array_filter($parts) as $unit => $value) {
-		$result[] = formatFloat($value, null, ZBX_UNITS_ROUNDOFF_SUFFIXED).$units[$unit];
+	foreach (array_filter($parts) as $part_unit => $part_value) {
+		$result[] = formatFloat($part_value, null, ZBX_UNITS_ROUNDOFF_SUFFIXED).$units[$part_unit];
 	}
 
 	return $result ? ($value < 0 ? '-' : '').implode(' ', $result) : '0';
@@ -1270,7 +1251,7 @@ function zbx_str2links($text) {
 				if ($pos != $start) {
 					$result[] = mb_substr($line, $start, $pos - $start);
 				}
-				$result[] = new CLink(CHTML::encode($match), $match);
+				$result[] = new CLink(CHtml::encode($match), $match);
 				$start = $pos + mb_strlen($match);
 			}
 		}
@@ -1341,7 +1322,7 @@ function make_sorting_header($obj, $tabfield, $sortField, $sortOrder, $link = nu
  * @param string   $number     Valid number in decimal or scientific notation.
  * @param int|null $precision  Max number of significant digits to take into account. Default: ZBX_FLOAT_DIG.
  * @param int|null $decimals   Max number of first non-zero decimals decimals to display. Default: 0.
- * @param bool     $exact      Display exaclty this number of decimals instead of first non-zeros.
+ * @param bool     $exact      Display exactly this number of decimals instead of first non-zeros.
  *
  * Note: $decimals must be less than $precision.
  *
@@ -1514,14 +1495,12 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 	else {
 		// url to redirect the user to after he logs in
 		$url = (new CUrl(!empty($_REQUEST['request']) ? $_REQUEST['request'] : ''))->removeArgument('sid');
-		$config = select_config();
 
-		if ($config['http_login_form'] == ZBX_AUTH_FORM_HTTP && $config['http_auth_enabled'] == ZBX_AUTH_HTTP_ENABLED
+		if (CAuthenticationHelper::get(CAuthenticationHelper::HTTP_LOGIN_FORM) == ZBX_AUTH_FORM_HTTP
+				&& CAuthenticationHelper::get(CAuthenticationHelper::HTTP_AUTH_ENABLED) == ZBX_AUTH_HTTP_ENABLED
 				&& (!CWebUser::isLoggedIn() || CWebUser::isGuest())) {
 			$redirect_to = (new CUrl('index_http.php'))->setArgument('request', $url->toString());
 			redirect($redirect_to->toString());
-
-			exit;
 		}
 
 		$url = urlencode($url->toString());
@@ -1531,7 +1510,9 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 			$data = [
 				'header' => _('Access denied'),
 				'messages' => [
-					_s('You are logged in as "%1$s".', CWebUser::$data['alias']).' '._('You have no permissions to access this page.'),
+					_s('You are logged in as "%1$s".',
+						CWebUser::$data['username']).' '._('You have no permissions to access this page.'
+					),
 					_('If you think this message is wrong, please consult your administrators about getting the necessary permissions.')
 				],
 				'buttons' => []
@@ -1542,8 +1523,9 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 				$data['buttons'][] = (new CButton('login', _('Login')))
 					->onClick('javascript: document.location = "index.php?request='.$url.'";');
 			}
-			$data['buttons'][] = (new CButton('back', _('Go to dashboard')))
-				->onClick('javascript: document.location = "zabbix.php?action=dashboard.view"');
+
+			$data['buttons'][] = (new CButton('back', _s('Go to "%1$s"', CMenuHelper::getFirstLabel())))
+				->onClick('javascript: document.location = "'.CMenuHelper::getFirstUrl().'"');
 		}
 		// if the user is not logged in - offer to login
 		else {
@@ -1567,7 +1549,8 @@ function access_deny($mode = ACCESS_DENY_OBJECT) {
 		else {
 			echo (new CView('general.warning', $data))->getOutput();
 		}
-		exit;
+		session_write_close();
+		exit();
 	}
 }
 
@@ -1594,8 +1577,7 @@ function detect_page_type($default = PAGE_TYPE_HTML) {
 	return $default;
 }
 
-function makeMessageBox($good, array $messages, $title = null, $show_close_box = true, $show_details = false)
-{
+function makeMessageBox($good, array $messages, $title = null, $show_close_box = true, $show_details = false) {
 	$class = $good ? ZBX_STYLE_MSG_GOOD : ZBX_STYLE_MSG_BAD;
 	$msg_details = null;
 	$link_details = null;
@@ -1626,12 +1608,16 @@ function makeMessageBox($good, array $messages, $title = null, $show_close_box =
 				$list->setAttribute('style', 'display: none;');
 			}
 		}
+
 		foreach ($messages as $message) {
 			foreach (explode("\n", $message['message']) as $message_part) {
 				$list->addItem($message_part);
 			}
 		}
-		$msg_details = (new CDiv())->addClass(ZBX_STYLE_MSG_DETAILS)->addItem($list);
+
+		$msg_details = (new CDiv())
+			->addClass(ZBX_STYLE_MSG_DETAILS)
+			->addItem($list);
 	}
 
 	if ($title !== null) {
@@ -1655,33 +1641,32 @@ function makeMessageBox($good, array $messages, $title = null, $show_close_box =
 }
 
 /**
- * Filters messages that can be displayed to user based on defines (see ZBX_SHOW_TECHNICAL_ERRORS) and user settings.
- *
- * @param array $messages	List of messages to filter.
+ * Filters messages that can be displayed to user based on CSettingsHelper::SHOW_TECHNICAL_ERRORS and user settings.
  *
  * @return array
  */
-function filter_messages(array $messages = []) {
-	if (!ZBX_SHOW_TECHNICAL_ERRORS && CWebUser::getType() != USER_TYPE_SUPER_ADMIN && !CWebUser::getDebugMode()) {
-		$filtered_messages = [];
-		$generic_exists = false;
+function filter_messages(): array {
+	if (!CSettingsHelper::getGlobal(CSettingsHelper::SHOW_TECHNICAL_ERRORS)
+			&& CWebUser::getType() != USER_TYPE_SUPER_ADMIN && !CWebUser::getDebugMode()) {
+		$messages = CMessageHelper::getMessages();
+		CMessageHelper::clear(false);
 
+		$generic_exists = false;
 		foreach ($messages as $message) {
-			if (array_key_exists('src', $message) && ($message['src'] === 'sql' || $message['src'] === 'php')) {
+			if ($message['type'] === CMessageHelper::MESSAGE_TYPE_ERROR
+					&& ($message['source'] === 'sql' || $message['source'] === 'php')) {
 				if (!$generic_exists) {
-					$message['message'] = _('System error occurred. Please contact Zabbix administrator.');
-					$filtered_messages[] = $message;
+					CMessageHelper::addError(_('System error occurred. Please contact Zabbix administrator.'));
 					$generic_exists = true;
 				}
 			}
 			else {
-				$filtered_messages[] = $message;
+				CMessageHelper::addSuccess($message['message']);
 			}
 		}
-		$messages = $filtered_messages;
 	}
 
-	return $messages;
+	return CMessageHelper::getMessages();
 }
 
 /**
@@ -1690,35 +1675,28 @@ function filter_messages(array $messages = []) {
  * @param  bool    $good            Parameter passed to makeMessageBox to specify message box style.
  * @param  string  $title           Message box title.
  * @param  bool    $show_close_box  Show or hide close button in error message box.
- * @global array   $ZBX_MESSAGES
  *
- * @return CDiv|null
+ * @return CTag|null
  */
-function getMessages($good = false, $title = null, $show_close_box = true) {
-	global $ZBX_MESSAGES;
-
-	$messages = (isset($ZBX_MESSAGES) && $ZBX_MESSAGES) ? filter_messages($ZBX_MESSAGES) : [];
+function getMessages(bool $good = false, string $title = null, bool $show_close_box = true): ?CTag {
+	$messages = get_and_clear_messages();
 
 	$message_box = ($title || $messages)
-		? makeMessageBox($good, $messages, $title, $show_close_box)
+		? makeMessageBox($good, $messages, $title, $show_close_box, !$good)
 		: null;
 
-	$ZBX_MESSAGES = [];
-
 	return $message_box;
 }
 
 function show_messages($good = false, $okmsg = null, $errmsg = null) {
-	global $page, $ZBX_MESSAGES, $ZBX_MESSAGES_PREPARED;
+	global $page, $ZBX_MESSAGES_PREPARED;
 
 	if (defined('ZBX_API_REQUEST')) {
 		return null;
 	}
 
 	$title = $good ? $okmsg : $errmsg;
-	$messages = isset($ZBX_MESSAGES) ? filter_messages($ZBX_MESSAGES) : [];
-
-	$ZBX_MESSAGES = [];
+	$messages = get_and_clear_messages();
 
 	if ($title === null && !$messages) {
 		return;
@@ -1783,7 +1761,6 @@ function show_messages($good = false, $okmsg = null, $errmsg = null) {
 
 			imageOut($canvas);
 			imagedestroy($canvas);
-
 			break;
 
 		default:
@@ -1814,7 +1791,7 @@ function show_messages($good = false, $okmsg = null, $errmsg = null) {
  * @return string|null  One or several HTML message boxes.
  */
 function get_prepared_messages(array $options = []): ?string {
-	global $ZBX_MESSAGES, $ZBX_MESSAGES_PREPARED;
+	global $ZBX_MESSAGES_PREPARED;
 
 	if (!is_array($ZBX_MESSAGES_PREPARED)) {
 		$ZBX_MESSAGES_PREPARED = [];
@@ -1829,7 +1806,11 @@ function get_prepared_messages(array $options = []): ?string {
 	// Process messages of the current request.
 
 	if ($options['with_current_messages']) {
-		show_messages();
+		show_messages(
+			CMessageHelper::getType() === CMessageHelper::MESSAGE_TYPE_SUCCESS,
+			CMessageHelper::getTitle(),
+			CMessageHelper::getTitle()
+		);
 
 		$messages_current = $ZBX_MESSAGES_PREPARED;
 		$restore_messages = [];
@@ -1837,11 +1818,11 @@ function get_prepared_messages(array $options = []): ?string {
 	}
 	else {
 		$messages_current = [];
-		$restore_messages = $ZBX_MESSAGES;
+		$restore_messages = CMessageHelper::getMessages();
 		$restore_messages_prepared = $ZBX_MESSAGES_PREPARED;
+		CMessageHelper::clear();
 	}
 
-	$ZBX_MESSAGES = [];
 	$ZBX_MESSAGES_PREPARED = [];
 
 	// Process authentication warning if user had unsuccessful authentication attempts.
@@ -1858,33 +1839,31 @@ function get_prepared_messages(array $options = []): ?string {
 			$failed_attempts
 		));
 
-		show_messages();
+		show_messages(
+			false, // Failed login can be only error message.
+			CMessageHelper::getTitle(),
+			CMessageHelper::getTitle()
+		);
 
 		CProfile::update('web.login.attempt.failed', 0, PROFILE_TYPE_INT);
 	}
 
 	$messages_authentication = $ZBX_MESSAGES_PREPARED;
-	$ZBX_MESSAGES = [];
 	$ZBX_MESSAGES_PREPARED = [];
 
 	// Process messages passed by the previous request.
 
-	if ($options['with_session_messages']
-			&& (CSession::keyExists('messageOk') || CSession::keyExists('messageError'))) {
-		if (CSession::keyExists('messages')) {
-			$ZBX_MESSAGES = CSession::getValue('messages');
-		}
+	if ($options['with_session_messages']) {
+		CMessageHelper::restoreScheduleMessages($messages_current);
 
-		if (CSession::keyExists('messageOk')) {
-			show_messages(true, CSession::getValue('messageOk'), null);
+		if (CMessageHelper::getTitle() !== null) {
+			show_messages(
+				CMessageHelper::getType() === CMessageHelper::MESSAGE_TYPE_SUCCESS,
+				CMessageHelper::getTitle(),
+				CMessageHelper::getTitle()
+			);
 		}
-		else {
-			show_messages(false, null, CSession::getValue('messageError'));
-		}
-
-		CSession::unsetValue(['messages', 'messageOk', 'messageError']);
 	}
-
 	$messages_session = $ZBX_MESSAGES_PREPARED;
 
 	// Create message boxes for all requested messages types in the correct order.
@@ -1896,31 +1875,28 @@ function get_prepared_messages(array $options = []): ?string {
 		)->toString();
 	}
 
-	$ZBX_MESSAGES = $restore_messages;
+	foreach ($restore_messages as $message) {
+		CMessageHelper::addMessage($message);
+	}
+
 	$ZBX_MESSAGES_PREPARED = $restore_messages_prepared;
 
 	return ($html === '') ? null : $html;
 }
 
-function show_message($msg) {
+function show_message(string $msg): void {
 	show_messages(true, $msg, '');
 }
 
-function show_error_message($msg) {
+function show_error_message(string $msg): void {
 	show_messages(false, '', $msg);
 }
 
-function info($msgs) {
-	global $ZBX_MESSAGES;
-
-	if (!isset($ZBX_MESSAGES)) {
-		$ZBX_MESSAGES = [];
-	}
-
+function info($msgs): void {
 	zbx_value2array($msgs);
 
 	foreach ($msgs as $msg) {
-		$ZBX_MESSAGES[] = ['type' => 'info', 'message' => $msg];
+		CMessageHelper::addSuccess($msg);
 	}
 }
 
@@ -1930,53 +1906,19 @@ function info($msgs) {
  * @param string | array $msg	Error message text.
  * @param string		 $src	The source of error message.
  */
-function error($msgs, $src = '') {
-	global $ZBX_MESSAGES;
-
-	if (!isset($ZBX_MESSAGES)) {
-		$ZBX_MESSAGES = [];
-	}
-
+function error($msgs, string $src = ''): void {
 	$msgs = zbx_toArray($msgs);
 
 	foreach ($msgs as $msg) {
-		$ZBX_MESSAGES[] = [
-			'type' => 'error',
-			'message' => $msg,
-			'src' => $src
-		];
+		CMessageHelper::addError($msg, $src);
 	}
 }
 
-/**
- * Add multiple errors under single header.
- *
- * @param array  $data
- * @param string $data['header']  common header for all error messages
- * @param array  $data['msgs']    array of error messages
- */
-function error_group($data) {
-	foreach (zbx_toArray($data['msgs']) as $msg) {
-		error($data['header'] . ' ' . $msg);
-	}
-}
+function get_and_clear_messages(): array {
+	$messages = filter_messages();
+	CMessageHelper::clear();
 
-function clear_messages($count = null) {
-	global $ZBX_MESSAGES;
-
-	if ($count != null) {
-		$result = [];
-
-		while ($count-- > 0) {
-			array_unshift($result, array_pop($ZBX_MESSAGES));
-		}
-	}
-	else {
-		$result = $ZBX_MESSAGES;
-		$ZBX_MESSAGES = [];
-	}
-
-	return $result ? filter_messages($result) : $result;
+	return $messages;
 }
 
 function fatal_error($msg) {
@@ -2022,15 +1964,20 @@ function get_status() {
 		'has_status' => false
 	];
 
-	$server = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT, ZBX_SOCKET_TIMEOUT, ZBX_SOCKET_BYTES_LIMIT);
-	$status['is_running'] = $server->isRunning(get_cookie(ZBX_SESSION_NAME));
+	$server = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT,
+		timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::CONNECT_TIMEOUT)),
+		timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::SOCKET_TIMEOUT)), ZBX_SOCKET_BYTES_LIMIT
+	);
+	$status['is_running'] = $server->isRunning(CSessionHelper::getId());
 
 	if ($status['is_running'] === false) {
 		return $status;
 	}
 
-	$server = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT, 15, ZBX_SOCKET_BYTES_LIMIT);
-	$server_status = $server->getStatus(get_cookie(ZBX_SESSION_NAME));
+	$server = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT,
+		timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::CONNECT_TIMEOUT)), 15, ZBX_SOCKET_BYTES_LIMIT
+	);
+	$server_status = $server->getStatus(CSessionHelper::getId());
 	$status['has_status'] = (bool) $server_status;
 
 	if ($server_status === false) {
@@ -2201,7 +2148,7 @@ function imageOut(&$image, $format = null) {
 
 	if ($page['type'] != PAGE_TYPE_IMAGE) {
 		$imageId = md5(strlen($imageSource));
-		CSession::setValue('image_id', [$imageId => $imageSource]);
+		CSessionHelper::set('image_id', [$imageId => $imageSource]);
 	}
 
 	switch ($page['type']) {
@@ -2220,22 +2167,10 @@ function imageOut(&$image, $format = null) {
 /**
  * Check if we have error messages to display.
  *
- * @global array $ZBX_MESSAGES
- *
  * @return bool
  */
 function hasErrorMesssages() {
-	global $ZBX_MESSAGES;
-
-	if (isset($ZBX_MESSAGES)) {
-		foreach ($ZBX_MESSAGES as $message) {
-			if ($message['type'] === 'error') {
-				return true;
-			}
-		}
-	}
-
-	return false;
+	return CMessageHelper::getType() === CMessageHelper::MESSAGE_TYPE_ERROR;
 }
 
 /**
@@ -2487,13 +2422,16 @@ function getTimeSelectorPeriod(array $options) {
 	$profileIdx2 = array_key_exists('profileIdx2', $options) ? $options['profileIdx2'] : null;
 
 	if ($profileIdx === null) {
-		$options['from'] = ZBX_PERIOD_DEFAULT_FROM;
-		$options['to'] = ZBX_PERIOD_DEFAULT_TO;
+		$options['from'] = 'now-'.CSettingsHelper::get(CSettingsHelper::PERIOD_DEFAULT);
+		$options['to'] = 'now';
 	}
 	elseif (!array_key_exists('from', $options) || !array_key_exists('to', $options)
 			|| $options['from'] === null || $options['to'] === null) {
-		$options['from'] = CProfile::get($profileIdx.'.from', ZBX_PERIOD_DEFAULT_FROM, $profileIdx2);
-		$options['to'] = CProfile::get($profileIdx.'.to', ZBX_PERIOD_DEFAULT_TO, $profileIdx2);
+		$options['from'] = CProfile::get($profileIdx.'.from',
+			'now-'.CSettingsHelper::get(CSettingsHelper::PERIOD_DEFAULT),
+			$profileIdx2
+		);
+		$options['to'] = CProfile::get($profileIdx.'.to', 'now', $profileIdx2);
 	}
 
 	$range_time_parser = new CRangeTimeParser();
@@ -2506,6 +2444,31 @@ function getTimeSelectorPeriod(array $options) {
 	return $options;
 }
 
+/**
+ * Get array of action statuses available for defined time range. For incorrect "from" or "to" all actions will be set
+ * to false.
+ *
+ * @param string $from      Relative or absolute time, cannot be null.
+ * @param string $to        Relative or absolute time, cannot be null.
+ *
+ * @return array
+ */
+function getTimeselectorActions($from, $to): array {
+	$ts_now = time();
+	$parser = new CRangeTimeParser();
+	$ts_from = ($parser->parse($from) !== CParser::PARSE_FAIL) ? $parser->getDateTime(true)->getTimestamp() : null;
+	$ts_to = ($parser->parse($to) !== CParser::PARSE_FAIL) ? $parser->getDateTime(false)->getTimestamp() : null;
+	$valid = ($ts_from !== null && $ts_to !== null);
+	$parser->parse('now-'.CSettingsHelper::get(CSettingsHelper::MAX_PERIOD));
+	$max_period = 1 + $ts_now - $parser->getDateTime(true)->getTimestamp();
+
+	return [
+		'can_zoomout' => ($valid && ($ts_to - $ts_from + 1 < $max_period)),
+		'can_decrement' => ($valid && ($ts_from > 0)),
+		'can_increment' => ($valid && ($ts_to < $ts_now - ZBX_MIN_PERIOD))
+	];
+}
+
 /**
  * Convert relative date range string to translated string. Function does not check is passed date range correct.
  *
@@ -2585,3 +2548,55 @@ function relativeDateToText($from, $to) {
 
 	return $from.' – '.$to;
 }
+
+/**
+ * Get human readable time period.
+ *
+ * @param int $seconds
+ *
+ * @return string
+ */
+function secondsToPeriod(int $seconds): string {
+	$hours = floor($seconds / 3600);
+	$seconds -= $hours * 3600;
+
+	$minutes = floor($seconds / 60);
+	$seconds -= $minutes * 60;
+
+	$period = ($hours > 0) ? _n('%1$s hour', '%1$s hours', $hours) : '';
+
+	if ($minutes > 0) {
+		if ($period !== '') {
+			$period .= ', ';
+		}
+		$period .= _n('%1$s minute', '%1$s minutes', $minutes);
+	}
+
+	if ($seconds > 0 || $period === '') {
+		if ($period !== '') {
+			$period .= ', ';
+		}
+		$period .= _n('%1$s second', '%1$s seconds', $seconds);
+	}
+
+	return $period;
+}
+
+/**
+ * Generates UUID version 4.
+ *
+ * @param string $seed   String to be hashed as md5 and used as UUID body.
+ *
+ * @return string
+ */
+function generateUuidV4($seed = '') {
+	$data = ($seed === '') ? random_bytes(16) : hex2bin(md5($seed));
+
+	// Set head of 7th byte to 0100 (0100xxxx)
+	$data[6] = chr(ord($data[6]) & 0x0f | 0x40);
+
+	// Set head of 9th byte to 10 (10xxxxxx)
+	$data[8] = chr(ord($data[8]) & 0x3f | 0x80);
+
+	return bin2hex($data);
+}
