@@ -185,7 +185,7 @@
 						label_after: '}',
 						grow: 'auto',
 						rows: 0,
-						maxlength: <?= (int) $data['preprocessing_script_maxlength'] ?>
+						maxlength: <?= (int) DB::getFieldLength('item_preproc', 'params') ?>
 					});
 
 				case '<?= ZBX_PREPROC_PROMETHEUS_PATTERN ?>':
@@ -255,8 +255,24 @@
 			}
 		}
 
-		var $preprocessing = $('#preprocessing'),
-			step_index = $preprocessing.find('li.sortable').length;
+		var $preprocessing = $('#preprocessing');
+
+		if ($preprocessing.length === 0) {
+			const prep_elem = document.querySelector('#preprocessing_div');
+
+			if (!prep_elem) {
+				return false;
+			}
+
+			let obj = prep_elem;
+			if (prep_elem.tagName === 'SPAN') {
+				obj = prep_elem.originalObject;
+			}
+
+			$preprocessing = $(obj.querySelector('#preprocessing'));
+		}
+
+		var step_index = $preprocessing.find('li.sortable').length;
 
 		$preprocessing.sortable({
 			disabled: $preprocessing.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').hasClass('<?= ZBX_STYLE_DISABLED ?>'),
@@ -286,12 +302,14 @@
 					$('#preproc_test_all').show();
 					$preprocessing
 						.sortable('disable')
-						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').addClass('<?= ZBX_STYLE_DISABLED ?>');
+						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>')
+						.addClass('<?= ZBX_STYLE_DISABLED ?>');
 				}
 				else if (sortable_count > 1) {
 					$preprocessing
 						.sortable('enable')
-						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>').removeClass('<?= ZBX_STYLE_DISABLED ?>');
+						.find('div.<?= ZBX_STYLE_DRAG_ICON ?>')
+						.removeClass('<?= ZBX_STYLE_DISABLED ?>');
 				}
 
 				updateTypeOptionsAvailability();
